@@ -147,6 +147,22 @@ void account_save_config_all(void)
 	prefs_account_save_config_all(account_list);
 }
 
+PrefsAccount *account_find_from_smtp_server(const gchar *address,
+					    const gchar *smtp_server)
+{
+	GList *cur;
+	PrefsAccount *ac;
+
+	for (cur = account_list; cur != NULL; cur = cur->next) {
+		ac = (PrefsAccount *)cur->data;
+		if (!strcmp2(address, ac->address) &&
+		    !strcmp2(smtp_server, ac->smtp_server))
+			return ac;
+	}
+
+	return NULL;
+}
+
 PrefsAccount *account_find_from_id(gint id)
 {
 	GList *cur;
