@@ -1,7 +1,7 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
  *
- * Copyright (c) 2000 by Alfons Hoogervorst <alfons@proteus.demon.nl>
+ * Copyright (c) 2000-2001 by Alfons Hoogervorst <alfons@proteus.demon.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -253,7 +253,8 @@ gint start_address_completion(void)
 		/* open the address book */
 		read_address_book();
 		/* merge the completion entry list into g_completion */
-		g_completion_add_items(g_completion, g_completion_list);
+		if (g_completion_list)
+			g_completion_add_items(g_completion, g_completion_list);
 	}
 	g_ref_count++;
 	LOG_MESSAGE("start_address_completion ref count %d\n", g_ref_count);
@@ -498,11 +499,12 @@ gint end_address_completion(void)
 
 
 /* address completion entry ui. the ui (completion list was inspired by galeon's
- * auto completion list. remaining things powered by sylpheed's completion engine.
+ * auto completion list). remaining things powered by sylpheed's completion engine.
  */
 
-#define ENTRY_DATA_TAB_HOOK	"tab_hook"
-#define WINDOW_DATA_COMPL_ENTRY	"compl_entry"
+#define ENTRY_DATA_TAB_HOOK	"tab_hook"			/* used to lookup entry */
+#define WINDOW_DATA_COMPL_ENTRY	"compl_entry"	/* used to store entry for compl. window */
+#define WINDOW_DATA_COMPL_CLIST "compl_clist"	/* used to store clist for compl. window */
 
 static void address_completion_mainwindow_set_focus	(GtkWindow   *window,
 							 GtkWidget   *widget,
@@ -518,7 +520,7 @@ static void address_completion_create_completion_window	(GtkEntry    *entry);
 static void completion_window_select_row(GtkCList	 *clist,
 					 gint		  row,
 					 gint		  col,
-					 GdkEventButton  *event,
+					 GdkEvent	 *event,
 					 GtkWidget	**completion_window);
 static gboolean completion_window_button_press
 					(GtkWidget	 *widget,
@@ -529,6 +531,61 @@ static gboolean completion_window_key_press
 					 GdkEventKey	 *event,
 					 GtkWidget	**completion_window);
 
+
+static void completion_window_advance_to_row(GtkCList *clist, gint row)
+{
+	g_return_if_fail(row < g_completion_count);
+	gtk_clist_select_row(clist, row, 0);
+}
+
+static void completion_window_advance_selection(GtkCList *clist, gboolean forward)
+{
+	int row;
+
+	g_return_if_fail(clist != NULL);
+	g_return_if_fail(clist->selection != NULL);
+
+	row = GPOINTER_TO_INT(clist->selection->data);
+
+	row = forward ? (row + 1) % g_completion_count :
+			(row - 1) < 0 ? g_completion_count - 1 : row - 1;
+
+	gtk_clist_freeze(clist);
+	completion_window_advance_to_row(clist, row);					
+	gtk_clist_thaw(clist);
+}
+
+/* completion_window_accept_selection() - accepts the current selection in the
+ * clist, and destroys the window */
+static void completion_window_accept_selection(GtkWidget **window,
+					       GtkCList *clist,
+					       GtkEntry *entry)
+{
+	gchar *address = NULL, *text = NULL;
+	gint   cursor_pos, row, col;
+
+	g_return_if_fail(window != NULL);
+	g_return_if_fail(*window != NULL);
+	g_return_if_fail(clist != NULL);
+	g_return_if_fail(entry != NULL);
+	g_return_if_fail(clist->selection != NULL);
+
+	col = 0;
+
+	/* FIXME: I believe it's acceptable to access the selection member directly  */
+	row = GPOINTER_TO_INT(clist->selection->data);
+
+	/* we just need the cursor position */
+	address = get_address_from_edit(entry, &cursor_pos);
+	gtk_clist_get_text(clist, row, col, &text);
+	replace_address_in_edit(entry, text, cursor_pos);
+	g_free(address);				
+
+	clear_completion_cache();
+	gtk_widget_destroy(*window);
+	*window = NULL;
+}
+
 /* should be called when creating the main window containing address
  * completion entries */
 void address_completion_start(GtkWidget *mainwindow)
@@ -541,8 +598,6 @@ void address_completion_start(GtkWidget *mainwindow)
 			   mainwindow);
 }
 
-/* register an entry which will become address completed
- */
 void address_completion_register_entry(GtkEntry *entry)
 {
 	g_return_if_fail(entry != NULL);
@@ -680,20 +735,23 @@ static void address_completion_create_completion_window(GtkEntry *entry_)
 	gtk_container_add(GTK_CONTAINER(completion_window), scroll);
 	gtk_container_add(GTK_CONTAINER(scroll), clist);
 
-	/* set the unique data so we can always get back the entry window to which 
-	 * this completion window has been attached */
+	/* set the unique data so we can always get back the entry and
+	 * clist window to which this completion window has been attached */
 	gtk_object_set_data(GTK_OBJECT(completion_window),
 			    WINDOW_DATA_COMPL_ENTRY, entry_);
+	gtk_object_set_data(GTK_OBJECT(completion_window),
+			    WINDOW_DATA_COMPL_CLIST, clist);
 
 	gtk_signal_connect(GTK_OBJECT(clist), "select_row",
 			   GTK_SIGNAL_FUNC(completion_window_select_row),
 			   &completion_window);
 
 	for (count = 0; count < get_completion_count(); count++) {
-		gchar *label = get_complete_address(count);
-		gchar *text[] = { label, NULL };
+		gchar *text[] = {NULL, NULL};
+
+		text[0] = get_complete_address(count);
 		gtk_clist_append(GTK_CLIST(clist), text);
-		g_free(label);
+		g_free(text[0]);
 	}
 
 	gdk_window_get_geometry(entry->window, &x, &y, &width, &height, &depth);
@@ -726,21 +784,39 @@ static void address_completion_create_completion_window(GtkEntry *entry_)
 			 GDK_BUTTON_RELEASE_MASK,
 			 NULL, NULL, GDK_CURRENT_TIME);
 	gtk_grab_add(completion_window);
-	gtk_widget_grab_focus(GTK_WIDGET(clist));
-	gtk_signal_emit_by_name(GTK_OBJECT(clist), "scroll_vertical",
-				GTK_SCROLL_STEP_FORWARD, 0);
+
+	/* this gets rid of the irritating focus rectangle that doesn't
+	 * follow the selection */
+	GTK_WIDGET_UNSET_FLAGS(clist, GTK_CAN_FOCUS);
+	gtk_clist_select_row(GTK_CLIST(clist), 1, 0);
 }
 
-/* row selection sends completed address to entry */
+
+/* row selection sends completed address to entry.
+ * note: event is NULL if selected by anything else than a mouse button. */
 static void completion_window_select_row(GtkCList *clist, gint row, gint col,
-					 GdkEventButton *event,
+					 GdkEvent *event,
 					 GtkWidget **completion_window)
 {
-	gchar *text;
-	gint cursor_pos;
-	gchar *address;
 	GtkEntry *entry;
 
+	/* first check if it's anything but a mouse event. Mouse events
+	 * accept the completion. Anything else is accepted by the
+	 * completion_window_key_press() */
+	if (!event) {
+		/* event == NULL if key press did the selection or just
+		 * event emitted with signal_emit_XXX(). This seems to 
+		 * be the case for the gtk versions I have seen */
+		return;
+	}
+
+	/* however, a future version of GTK might pass the event type
+	 * that triggered the select_row. since this event handler
+	 * only wants mouse clicks, we check for that. */
+	if (event->type != GDK_BUTTON_RELEASE) {
+		return;
+	}
+
 	g_return_if_fail(completion_window != NULL);
 	g_return_if_fail(*completion_window != NULL);
 
@@ -748,18 +824,12 @@ static void completion_window_select_row(GtkCList *clist, gint row, gint col,
 					      WINDOW_DATA_COMPL_ENTRY));
 	g_return_if_fail(entry != NULL);
 
-	/* if selected by a click */
-	address = get_address_from_edit(entry, &cursor_pos);
-	gtk_clist_get_text(clist, row, col, &text);
-	replace_address_in_edit(entry, text, cursor_pos);
-	g_free(address);				
-
-	clear_completion_cache();
-	gtk_widget_destroy(*completion_window);
-	*completion_window = NULL;
+	completion_window_accept_selection(completion_window, clist, entry);	
 }
 
-/* if mouse button pressed see if it's in the completion list */
+/* completion_window_button_press() - check is mouse click is anywhere
+ * else (not in the completion window). in that case the completion
+ * window is destroyed, and the original prefix is restored */
 static gboolean completion_window_button_press(GtkWidget *widget,
 					       GdkEventButton *event,
 					       GtkWidget **completion_window)
@@ -810,38 +880,45 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	GtkWidget *entry;
 	gchar *prefix;
 	gint cursor_pos;
+	GtkWidget *clist;
 
 	g_return_val_if_fail(completion_window != NULL, FALSE);
 	g_return_val_if_fail(*completion_window != NULL, FALSE);
 
 	entry = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(*completion_window),
 					       WINDOW_DATA_COMPL_ENTRY));
+	clist = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(*completion_window),
+					       WINDOW_DATA_COMPL_CLIST));
 	g_return_val_if_fail(entry != NULL, FALSE);
 
 	/* allow keyboard navigation in the alternatives clist */
 	if (event->keyval == GDK_Up || event->keyval == GDK_Down ||
-	    event->keyval == GDK_Page_Up ||  event->keyval == GDK_Page_Down)
+	    event->keyval == GDK_Page_Up || event->keyval == GDK_Page_Down) {
+		completion_window_advance_selection
+			(GTK_CLIST(clist),
+			 event->keyval == GDK_Down ||
+			 event->keyval == GDK_Page_Down ? TRUE : FALSE);
 		return FALSE;
+	}		
 
-	/* also make tab / shift tab go to next previous completion entry */
+	/* also make tab / shift tab go to next previous completion entry. we're
+	 * changing the key value */
 	if (event->keyval == GDK_Tab || event->keyval == GDK_ISO_Left_Tab) {
 		event->keyval = (event->state & GDK_SHIFT_MASK)
 			? GDK_Up : GDK_Down;
 		/* need to reset shift state if going up */
 		if (event->state & GDK_SHIFT_MASK)
 			event->state &= ~GDK_SHIFT_MASK;
-
+		completion_window_advance_selection(GTK_CLIST(clist), 
+			event->keyval == GDK_Down ? TRUE : FALSE);
 		return FALSE;
 	}
 
-	/* make return act like a space */
-	if (event->keyval == GDK_Return) {
-		event->keyval = GDK_space;
-		return FALSE;
-	}
-
-	/* space selects row */
-	if (event->keyval == GDK_space) {
+	/* look for presses that accept the selection */
+	if (event->keyval == GDK_Return || event->keyval == GDK_space) {
+		completion_window_accept_selection(completion_window,
+						   GTK_CLIST(clist),
+						   GTK_ENTRY(entry));
 		return FALSE;
 	}
 
