@@ -24,6 +24,7 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 
+#include "main.h"
 #include "filesel.h"
 #include "manage_window.h"
 #include "gtkutils.h"
@@ -51,7 +52,7 @@ gchar *filesel_select_file(const gchar *title, const gchar *file)
 	}
 
 	if (!cwd)
-		cwd = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, NULL);
+		cwd = g_strconcat(startup_dir, G_DIR_SEPARATOR_S, NULL);
 
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), cwd);
 
