@@ -442,11 +442,11 @@ gchar *folder_item_get_path(FolderItem *item)
 			path = g_strdup(folder_path);
 	} else {
 		if (item->path)
-			path = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S,
+			path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 					   folder_path, G_DIR_SEPARATOR_S,
 					   item->path, NULL);
 		else
-			path = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S,
+			path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 					   folder_path, NULL);
 	}
 
@@ -693,6 +693,7 @@ static void folder_init(Folder *folder, FolderType type, const gchar *name)
 		folder->remove_all_msg      = imap_remove_all_msg;
 		folder->scan                = imap_scan_folder;
 		folder->create_folder       = imap_create_folder;
+		folder->remove_folder       = imap_remove_folder;
 		break;
 	case F_NEWS:
 		folder->get_msg_list = news_get_article_list;
