@@ -1,4 +1,23 @@
-/* foldersel.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+#include "defs.h"
 
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
@@ -11,18 +30,22 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtksignal.h>
+#include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
+#include <errno.h>
 
+#include "intl.h"
 #include "main.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "foldersel.h"
 #include "alertpanel.h"
 #include "manage_window.h"
+#include "folder.h"
 
 #include "pixmaps/dir-close.xpm"
 #include "pixmaps/dir-open.xpm"
@@ -37,14 +60,13 @@ static GtkWidget *ctree;
 static GtkWidget *entry;
 static GtkWidget *ok_button;
 static GtkWidget *cancel_button;
-static gchar *folder;
+
+static FolderItem *folder_item;
 
 static gboolean cancelled;
 
 static void foldersel_create(void);
 static void foldersel_init(void);
-static void foldersel_free_str(GtkCTree *ctree, GtkCTreeNode *node,
-			       gpointer data);
 static void foldersel_set_tree(void);
 static void foldersel_selected(GtkCList *clist, gint row, gint column,
 			       GdkEvent *event, gpointer data);
@@ -54,7 +76,7 @@ static void foldersel_cancel(GtkButton *button, gpointer data);
 static void foldersel_activated(void);
 static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
 
-gchar *foldersel_folder_sel(const gchar *default_folder)
+FolderItem *foldersel_folder_sel(const gchar *default_folder)
 {
 	GtkCTreeNode *node;
 
@@ -67,16 +89,22 @@ gchar *foldersel_folder_sel(const gchar *default_folder)
 
 	foldersel_set_tree();
 
-	if (folder) {
-		debug_print(_("Previously selected folder: %s\n"), folder);
-		node = gtk_ctree_find_by_row_data_custom
-			(GTK_CTREE(ctree), NULL, folder,
-			 (GCompareFunc)path_cmp);
-		if (node)
-			gtk_ctree_select(GTK_CTREE(ctree), node);
+	if (folder_item) {
+		node = gtk_ctree_find_by_row_data
+			(GTK_CTREE(ctree), NULL, folder_item);
+		if (node) {
+			gint row;
+
+			row = gtkut_ctree_get_nth_from_node
+				(GTK_CTREE(ctree), node);
+			gtk_clist_select_row(GTK_CLIST(ctree), row, -1);
+			gtkut_clist_set_focus_row(GTK_CLIST(ctree), row);
+			gtk_ctree_node_moveto(GTK_CTREE(ctree), node, -1,
+					      0.5, 0);
+		}
 	}
 	gtk_widget_grab_focus(ok_button);
-	gtk_widget_grab_focus(entry);
+	gtk_widget_grab_focus(ctree);
 
 	cancelled = FALSE;
 
@@ -84,14 +112,11 @@ gchar *foldersel_folder_sel(const gchar *default_folder)
 
 	gtk_widget_hide(window);
 	gtk_entry_set_text(GTK_ENTRY(entry), "");
-	gtk_ctree_pre_recursive(GTK_CTREE(ctree), NULL,
-				foldersel_free_str, NULL);
 	gtk_clist_clear(GTK_CLIST(ctree));
 
-	if (!cancelled) {
-		debug_print("folder: %s\n", folder);
-		return folder;
-	} else
+	if (!cancelled && folder_item && folder_item->path)
+		return folder_item;
+	else
 		return NULL;
 }
 
@@ -112,6 +137,10 @@ static void foldersel_create(void)
 			   GTK_SIGNAL_FUNC(foldersel_cancel), NULL);
 	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
 			   GTK_SIGNAL_FUNC(key_pressed), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "focus_in_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "focus_out_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);
 
 	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -127,6 +156,8 @@ static void foldersel_create(void)
 	gtk_container_add(GTK_CONTAINER(scrolledwin), ctree);
 	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
 	gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_DOTTED);
+	gtk_ctree_set_expander_style(GTK_CTREE(ctree),
+				     GTK_CTREE_EXPANDER_SQUARE);
 	gtk_ctree_set_indent(GTK_CTREE(ctree), CTREE_INDENT);
 	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(ctree)->column[0].button,
 			       GTK_CAN_FOCUS);
@@ -163,67 +194,54 @@ static void foldersel_init(void)
 		      DIRECTORY_OPEN_XPM);
 }
 
-static void foldersel_free_str(GtkCTree *ctree, GtkCTreeNode *node,
-			       gpointer data)
+static gboolean foldersel_gnode_func(GtkCTree *ctree, guint depth,
+				     GNode *gnode, GtkCTreeNode *cnode,
+				     gpointer data)
 {
-	gchar *ptr = gtk_ctree_node_get_row_data(ctree, node);
+	FolderItem *item = FOLDER_ITEM(gnode->data);
 
-	g_free(ptr);
+	gtk_ctree_node_set_row_data(ctree, cnode, item);
+	gtk_ctree_set_node_info(ctree, cnode, item->name,
+				FOLDER_SPACING,
+				folderxpm, folderxpmmask,
+				folderopenxpm, folderopenxpmmask,
+				FALSE, FALSE);
+
+	return TRUE;
+}
+
+static void foldersel_expand_func(GtkCTree *ctree, GtkCTreeNode *node,
+				  gpointer data)
+{
+	if (GTK_CTREE_ROW(node)->children)
+		gtk_ctree_expand(ctree, node);
 }
 
 static void foldersel_set_tree(void)
 {
-	FILE *fp;
-	gchar dir[MAXPATHLEN], parent[MAXPATHLEN];
-	gchar *p;
-	gchar *fcache;
-	gchar *text[1] = {"Mailbox"};
-	GtkCTreeNode *tnode, *node;
+	Folder *folder;
+	GtkCTreeNode *node;
+	GList *list;
 
-	CHDIR_RETURN_IF_FAIL(maildir);
-
-	fcache = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
-			     G_DIR_SEPARATOR_S, MAIL_FOLDER_LIST, NULL);
-	if ((fp = fopen(fcache, "r")) == NULL) {
-		fprintf(stderr, "%s: ", fcache);
-		perror("fopen");
-		g_free(fcache);
-		return;
-	}
-	g_free(fcache);
+	list = folder_get_list();
 
 	gtk_clist_freeze(GTK_CLIST(ctree));
 
-	tnode = gtk_ctree_insert_node(GTK_CTREE(ctree), NULL, NULL,
-				      text, FOLDER_SPACING,
-				      folderxpm, folderxpmmask,
-				      folderopenxpm, folderopenxpmmask,
-				      FALSE, TRUE);
+	for (; list != NULL; list = list->next) {
+		folder = FOLDER(list->data);
+		g_return_if_fail(folder != NULL);
 
-	while (fgets(dir, sizeof(dir), fp) != NULL) {
-		dir[strlen(dir) - 1] = '\0';
-		/* get parent directory */
-		if ((p = strrchr(dir, '/')) != NULL) {
-			strncpy(parent, dir, (size_t)(p - dir));
-			parent[p - dir] = '\0';
-			node = gtk_ctree_find_by_row_data_custom
-				(GTK_CTREE(ctree), tnode, parent,
-				 (GCompareFunc)strcmp2);
-			gtk_ctree_expand(GTK_CTREE(ctree), node);
-		} else
-			node = tnode;
-		text[0] = g_basename(dir);
-		node = gtk_ctree_insert_node
-			(GTK_CTREE(ctree), node, NULL, text, FOLDER_SPACING,
-			 folderxpm, folderxpmmask,
-			 folderopenxpm, folderopenxpmmask, FALSE, FALSE);
-		gtk_ctree_node_set_row_data(GTK_CTREE(ctree), node,
-					    g_strdup(dir));
-		fgets(dir, sizeof(dir), fp);
+		if (folder->type != F_MH) continue;
+
+		node = gtk_ctree_insert_gnode(GTK_CTREE(ctree), NULL, NULL,
+					      folder->node,
+					      foldersel_gnode_func,
+					      NULL);
+		gtk_ctree_sort_recursive(GTK_CTREE(ctree), node);
+		gtk_ctree_pre_recursive(GTK_CTREE(ctree), node,
+					foldersel_expand_func,
+					NULL);
 	}
-	fclose(fp);
-
-	gtk_ctree_sort_recursive(GTK_CTREE(ctree), tnode);
 
 	gtk_clist_thaw(GTK_CLIST(ctree));
 }
@@ -231,11 +249,12 @@ static void foldersel_set_tree(void)
 static void foldersel_selected(GtkCList *clist, gint row, gint column,
 			       GdkEvent *event, gpointer data)
 {
-	gchar *dir;
+	FolderItem *item;
 	GdkEventButton *ev = (GdkEventButton *)event;
 
-	dir = gtk_clist_get_row_data(clist, row);
-	if (dir) gtk_entry_set_text(GTK_ENTRY(entry), dir);
+	item = gtk_clist_get_row_data(clist, row);
+	if (item) gtk_entry_set_text(GTK_ENTRY(entry),
+				     item->path ? item->path : "");
 
 	if (ev && GDK_2BUTTON_PRESS == ev->type)
 		gtk_button_clicked(GTK_BUTTON(ok_button));
@@ -243,42 +262,12 @@ static void foldersel_selected(GtkCList *clist, gint row, gint column,
 
 static void foldersel_ok(GtkButton *button, gpointer data)
 {
-	gchar *dir;
-	struct stat s;
+	GList *list;
 
-	dir = gtk_entry_get_text(GTK_ENTRY(entry));
-	if (!dir || *dir == '\0') return;
-
-	if (stat(dir, &s) < 0) {
-		g_print("stat: ");
-		perror(dir);
-		if (alertpanel(_("New directory"),
-			       _("The directory not found. Create it?"),
-			       _("Yes"), _("No"), NULL) == G_ALERTDEFAULT) {
-			if (mkdir(dir, S_IRWXU) < 0) {
-				perror("mkdir");
-				alertpanel(_("Error"),
-					   _("Can't create directory."),
-					   _("OK"), NULL, NULL);
-				return;
-			}
-			if (chmod(dir, S_IRWXU) < 0) {
-				fprintf(stderr, "%s: ", dir);
-				perror("chmod");
-			}
-		} else
-			return;
-	} else {
-		if (!S_ISDIR(s.st_mode)) {
-			alertpanel(_("Error"),
-				   _("Selected name isn't a directory."),
-				   _("OK"), NULL, NULL);
-			return;
-		}
-	}
-
-	g_free(folder);
-	folder = g_strdup(dir);
+	list = GTK_CLIST(ctree)->selection;
+	if (list)
+		folder_item = gtk_ctree_node_get_row_data
+			(GTK_CTREE(ctree), GTK_CTREE_NODE(list->data));
 
 	gtk_main_quit();
 }
@@ -296,8 +285,6 @@ static void foldersel_activated(void)
 
 static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event && event->keyval == GDK_Escape) {
-		cancelled = TRUE;
-		gtk_main_quit();
-	}
+	if (event && event->keyval == GDK_Escape)
+		foldersel_cancel(NULL, NULL);
 }
