@@ -672,14 +672,24 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 	style = gtk_style_copy(prev_style);
 
 	if (style) {
-		/* if unread messages exist, print with bold font */
-		if (item->unread > 0 && boldfont)
+		gboolean use_bold, use_color;
+
+		if (item->stype == F_QUEUE) {
+			/* highlight queue folder if there are any messages */
+			use_bold = use_color = (item->total > 0);
+		} else {
+			/* if unread messages exist, print with bold font */
+			use_bold = (item->unread > 0);
+			/* if new messages exist, print with colored letter */
+			use_color = (item->new > 0);
+		}
+
+		if (use_bold && boldfont)
 			style->font = boldfont;
 		else
 			style->font = ctree_style->font;
 
-		/* if new messages exist, print with colored letter */
-		if (item->new > 0) {
+		if (use_color) {
 			style->fg[GTK_STATE_NORMAL]   = folderview->color_new;
 			style->fg[GTK_STATE_SELECTED] = folderview->color_new;
 		} else {
@@ -1395,12 +1405,19 @@ static void folderview_rm_imap_folder_cb(FolderView *folderview, guint action,
 	g_free(message);
 	if (avalue != G_ALERTDEFAULT) return;
 
-	if (folderview->opened == folderview->selected) {
+	if (item->folder->remove_folder(item->folder, item) < 0) {
+		g_warning(_("can't remove folder `%s'\n"), item->path);
+		return;
+	}
+
+	if (folderview->opened == folderview->selected ||
+	    gtk_ctree_is_ancestor(ctree,
+				  folderview->selected,
+				  folderview->opened)) {
 		summary_clear_all(folderview->summaryview);
 		folderview->opened = NULL;
 	}
 
-	folder_item_remove(item);
 	gtk_ctree_remove_node(ctree, folderview->selected);
 	folder_write_list();
 }
