@@ -151,6 +151,7 @@ void headerview_init(HeaderView *headerview)
 	SET_FONT_STYLE(subject_header_label);
 
 	headerview_clear(headerview);
+	headerview_set_visibility(headerview, prefs_common.display_header_pane);
 
 #if HAVE_LIBCOMPFACE
 	{
@@ -255,6 +256,14 @@ void headerview_clear(HeaderView *headerview)
 	}
 }
 
+void headerview_set_visibility(HeaderView *headerview, gboolean visibility)
+{
+	if (visibility)
+		gtk_widget_show(headerview->hbox);
+	else
+		gtk_widget_hide(headerview->hbox);
+}
+
 void headerview_destroy(HeaderView *headerview)
 {
 	g_free(headerview);
