@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ static void import_filesel_cb(GtkWidget *widget, gpointer data);
 static void import_destsel_cb(GtkWidget *widget, gpointer data);
 static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
 
-gint import_mbox(void)
+gint import_mbox(FolderItem *default_dest)
 {
 	gint ok = 0;
 
@@ -73,6 +73,13 @@ gint import_mbox(void)
 	else
 		gtk_widget_show(window);
 
+	gtk_entry_set_text(GTK_ENTRY(file_entry), "");
+	if (default_dest && default_dest->path)
+		gtk_entry_set_text(GTK_ENTRY(dest_entry), default_dest->path);
+	else
+		gtk_entry_set_text(GTK_ENTRY(dest_entry), "");
+	gtk_widget_grab_focus(file_entry);
+
 	manage_window_set_transient(GTK_WINDOW(window));
 
 	gtk_main();
@@ -107,6 +114,8 @@ gint import_mbox(void)
 static void import_create(void)
 {
 	GtkWidget *vbox;
+	GtkWidget *hbox;
+	GtkWidget *desc_label;
 	GtkWidget *table;
 	GtkWidget *file_label;
 	GtkWidget *dest_label;
@@ -114,11 +123,10 @@ static void import_create(void)
 
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
 	gtk_window_set_title(GTK_WINDOW(window), _("Import"));
-	gtk_widget_set_usize(window, 450, -1);
-	gtk_container_set_border_width(GTK_CONTAINER(window), 4);
+	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
+	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
 	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
 			   GTK_SIGNAL_FUNC(import_cancel_cb), NULL);
 	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
@@ -128,14 +136,23 @@ static void import_create(void)
 	gtk_signal_connect(GTK_OBJECT(window), "focus_out_event",
 			   GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);
 
-	vbox = gtk_vbox_new(FALSE, 8);
+	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
+
+	desc_label = gtk_label_new
+		(_("Specify target mbox file and destination folder."));
+	gtk_box_pack_start(GTK_BOX(hbox), desc_label, FALSE, FALSE, 0);
+
 	table = gtk_table_new(2, 3, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 8);
-	gtk_table_set_row_spacings(GTK_TABLE(table), 12);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
+	gtk_widget_set_usize(table, 420, -1);
 
 	file_label = gtk_label_new(_("Importing file:"));
 	gtk_table_attach(GTK_TABLE(table), file_label, 0, 1, 0, 1,
@@ -185,13 +202,15 @@ static void import_create(void)
 static void import_ok_cb(GtkWidget *widget, gpointer data)
 {
 	import_ack = TRUE;
-	gtk_main_quit();
+	if (gtk_main_level() > 1)
+		gtk_main_quit();
 }
 
 static void import_cancel_cb(GtkWidget *widget, gpointer data)
 {
 	import_ack = FALSE;
-	gtk_main_quit();
+	if (gtk_main_level() > 1)
+		gtk_main_quit();
 }
 
 static void import_filesel_cb(GtkWidget *widget, gpointer data)
