@@ -77,8 +77,11 @@ struct _Pop3State
 	gchar *pass;
 	gint count;
 	gint new;
-	gint bytes;
+	gint total_bytes;
 	gint cur_msg;
+	gint cur_msg_bytes;
+	gint cur_total_bytes;
+	gint *sizes;
 
 	/* UIDL */
 	GHashTable *id_table;
