@@ -124,7 +124,7 @@ void main_window_separation_change	(MainWindow	*mainwin,
 					 SeparateType	 type);
 void main_window_get_size		(MainWindow	*mainwin);
 void main_window_get_position		(MainWindow	*mainwin);
-void main_window_clean_trash		(MainWindow	*mainwin,
+void main_window_empty_trash		(MainWindow	*mainwin,
 					 gboolean	 confirm);
 void main_window_add_mailbox		(MainWindow	*mainwin);
 
