@@ -226,7 +226,8 @@ gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 		    dest->stype == F_QUEUE  ||
 		    dest->stype == F_DRAFT  ||
 		    dest->stype == F_TRASH)
-			MSG_UNSET_FLAGS(newmsginfo.flags, MSG_NEW|MSG_UNREAD);
+			MSG_UNSET_FLAGS(newmsginfo.flags,
+					MSG_NEW|MSG_UNREAD|MSG_DELETED);
 
 		procmsg_write_flags(&newmsginfo, fp);
 		fclose(fp);
@@ -308,7 +309,7 @@ gint mh_move_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 			    dest->stype == F_DRAFT  ||
 			    dest->stype == F_TRASH)
 				MSG_UNSET_FLAGS(newmsginfo.flags,
-						MSG_NEW|MSG_UNREAD);
+						MSG_NEW|MSG_UNREAD|MSG_DELETED);
 
 			procmsg_write_flags(&newmsginfo, fp);
 		}
@@ -381,7 +382,8 @@ gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 		    dest->stype == F_QUEUE  ||
 		    dest->stype == F_DRAFT  ||
 		    dest->stype == F_TRASH)
-			MSG_UNSET_FLAGS(newmsginfo.flags, MSG_NEW|MSG_UNREAD);
+			MSG_UNSET_FLAGS(newmsginfo.flags,
+					MSG_NEW|MSG_UNREAD|MSG_DELETED);
 		procmsg_write_flags(&newmsginfo, fp);
 		fclose(fp);
 	}
@@ -453,7 +455,7 @@ gint mh_copy_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 			    dest->stype == F_DRAFT  ||
 			    dest->stype == F_TRASH)
 				MSG_UNSET_FLAGS(newmsginfo.flags,
-						MSG_NEW|MSG_UNREAD);
+						MSG_NEW|MSG_UNREAD|MSG_DELETED);
 			procmsg_write_flags(&newmsginfo, fp);
 		}
 	}
@@ -630,7 +632,7 @@ gint mh_create_tree(Folder *folder)
 
 	g_return_val_if_fail(folder != NULL, -1);
 
-	CHDIR_RETURN_VAL_IF_FAIL(g_get_home_dir(), -1);
+	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), -1);
 	rootpath = LOCAL_FOLDER(folder)->rootpath;
 	MAKE_DIR_IF_NOT_EXIST(rootpath);
 	CHDIR_RETURN_VAL_IF_FAIL(rootpath, -1);
@@ -883,6 +885,30 @@ static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 	return msginfo;
 }
 
+static gboolean mh_is_maildir_one(const gchar *path, const gchar *dir)
+{
+	gchar *entry;
+	gboolean result;
+
+	entry = g_strconcat(path, G_DIR_SEPARATOR_S, dir, NULL);
+	result = is_dir_exist(entry);
+	g_free(entry);
+
+	return result;
+}
+
+/*
+ * check whether PATH is a Maildir style mailbox.
+ * This is the case if the 3 subdir: new, cur, tmp are existing.
+ * This functon assumes that entry is an directory
+ */
+static gboolean mh_is_maildir(const gchar *path)
+{
+	return mh_is_maildir_one(path, "new") &&
+	       mh_is_maildir_one(path, "cur") &&
+	       mh_is_maildir_one(path, "tmp");
+}
+
 static void mh_scan_tree_recursive(FolderItem *item)
 {
 	DIR *dp;
@@ -925,6 +951,11 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		if (S_ISDIR(s.st_mode)) {
 			FolderItem *new_item;
 
+			if (mh_is_maildir(entry)) {
+				g_free(entry);
+				continue;
+			}
+
 			new_item = folder_item_new(d->d_name, entry);
 			folder_item_append(item, new_item);
 			if (!item->path) {
