@@ -46,8 +46,9 @@ static gboolean pass_ack;
 
 static void passphrase_ok_cb(GtkWidget *widget, gpointer data);
 static void passphrase_cancel_cb(GtkWidget *widget, gpointer data);
-static void passphrase_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
-static GtkWidget *create_description ( const gchar *desc );
+static void passphrase_key_pressed(GtkWidget *widget, GdkEventKey *event,
+				   gpointer data);
+static GtkWidget *create_description (const gchar *desc);
 
 void
 gpgmegtk_set_passphrase_grab (gint yes)
@@ -124,18 +125,20 @@ gpgmegtk_passphrase_mbox (const gchar *desc)
 
     gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
                        GTK_SIGNAL_FUNC(passphrase_ok_cb), NULL);
+    gtk_signal_connect(GTK_OBJECT(pass_entry), "activate",
+                       GTK_SIGNAL_FUNC(passphrase_ok_cb), NULL);
     gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
                        GTK_SIGNAL_FUNC(passphrase_cancel_cb), NULL);
-    
-    if ( grab_all )
+
+    if (grab_all)
         gtk_object_set (GTK_OBJECT(window), "type", GTK_WINDOW_POPUP, NULL);
     gtk_window_set_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-    if ( grab_all )   
+    if (grab_all)   
         gtk_window_set_policy (GTK_WINDOW(window), FALSE, FALSE, TRUE);
     
     gtk_widget_show_all(window);
-    
-    if ( grab_all ) {
+
+    if (grab_all) {
         XGrabServer(GDK_DISPLAY());
         if ( gdk_pointer_grab ( window->window, TRUE, 0,
                                 NULL, NULL, GDK_CURRENT_TIME)) {
@@ -155,7 +158,7 @@ gpgmegtk_passphrase_mbox (const gchar *desc)
 
     gtk_main();
 
-    if ( grab_all ) {
+    if (grab_all) {
         XUngrabServer (GDK_DISPLAY());
         gdk_pointer_ungrab (GDK_CURRENT_TIME);
         gdk_keyboard_ungrab (GDK_CURRENT_TIME);
@@ -167,7 +170,7 @@ gpgmegtk_passphrase_mbox (const gchar *desc)
         if (the_passphrase) /* Hmmm: Do we really need this? */
             the_passphrase = g_strdup (the_passphrase);
     }
-    gtk_widget_destroy ( window );
+    gtk_widget_destroy (window);
 
     return the_passphrase;
 }
@@ -196,17 +199,18 @@ passphrase_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
 }
 
 static gint 
-linelen ( const gchar *s )
+linelen (const gchar *s)
 {
     gint i;
 
-    for ( i=0; *s && *s != '\n'; s++, i++ )
+    for (i = 0; *s && *s != '\n'; s++, i++)
         ;
+
     return i;
 }
 
 static GtkWidget *
-create_description ( const gchar *desc )
+create_description (const gchar *desc)
 {
     const gchar *cmd=NULL, *uid=NULL, *info=NULL;
     gchar *buf;
@@ -214,27 +218,26 @@ create_description ( const gchar *desc )
 
     cmd = desc;
     uid = strchr (cmd, '\n');
-    if ( uid ) {
+    if (uid) {
         info = strchr (++uid, '\n');
         if (info )
             info++;
     }
 
-    if ( !uid )
+    if (!uid)
         uid = _("[no user id]");
-    if ( !info )
+    if (!info)
         info = "";
 
-    buf = g_strdup_printf ( _("%sPlease enter the passphrase for:\n\n"
-                            "  %.*s  \n"
-                            "(%.*s)\n"),
-                            !strncmp (cmd, "TRY_AGAIN", 9 ) ?
-                            _("Bad passphrase! Try again...\n\n") : "",
-                            linelen (uid), uid, linelen (info), info );
+    buf = g_strdup_printf (_("%sPlease enter the passphrase for:\n\n"
+                           "  %.*s  \n"
+                           "(%.*s)\n"),
+                           !strncmp (cmd, "TRY_AGAIN", 9 ) ?
+                           _("Bad passphrase! Try again...\n\n") : "",
+                           linelen (uid), uid, linelen (info), info);
 
-
-    label = gtk_label_new ( buf );
+    label = gtk_label_new (buf);
     g_free (buf);
+
     return label;
 }
-
