@@ -20,6 +20,10 @@
 #ifndef __PREFS_ACCOUNT_H__
 #define __PREFS_ACCOUNT_H__
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <glib.h>
 
 typedef struct _PrefsAccount	PrefsAccount;
@@ -35,6 +39,14 @@ typedef enum {
 	A_LOCAL
 } RecvProtocol;
 
+#if USE_GPGME
+typedef enum {
+	SIGN_KEY_DEFAULT,
+	SIGN_KEY_BY_FROM,
+	SIGN_KEY_CUSTOM
+} SignKeyType;
+#endif /* USE_GPGME */
+
 struct _PrefsAccount
 {
 	gchar *account_name;
@@ -79,6 +91,12 @@ struct _PrefsAccount
 	/* Compose */
 	gchar *sig_path;
 
+#if USE_GPGME
+	/* Privacy */
+	SignKeyType sign_key;
+	gchar *sign_key_id;
+#endif /* USE_GPGME */
+
 	/* Advanced */
 	gboolean  set_smtpport;
 	gushort   smtpport;
