@@ -20,6 +20,10 @@
 #ifndef __PREFS_COMMON_H__
 #define __PREFS_COMMON_H__
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <glib.h>
 
 #include "mainwindow.h"
@@ -61,36 +65,19 @@ struct _PrefsCommon
 
 	/* Display */
 	gchar *widgetfont;
-	gchar *messagefont;
+	gchar *textfont;
 	gboolean display_folder_unread;
-	gboolean enable_color;
-	gint quote_level1_col;
-	gint quote_level2_col;
-	gint quote_level3_col;
-	gint uri_col;
-	gushort sig_col;
-	gboolean recycle_quote_colors;
 	ToolbarStyle toolbar_style;
 	gboolean show_statusbar;
 	gboolean trans_hdr;
 	gboolean enable_thread;
 	gboolean enable_hscrollbar;
-	gboolean conv_mb_alnum;
-	gboolean head_space;
 	gboolean swap_from;
-	gint line_space;
-	gboolean display_header;
-
+	gchar *date_format;
 
 	/* Filtering */
 	GSList *fltlist;
 
-	/* MIME viewer */
-	gchar *mime_image_viewer;
-	gchar *mime_audio_player;
-
-	gchar *force_charset;
-
 	gboolean show_mark;
 	gboolean show_unread;
 	gboolean show_mime;
@@ -136,6 +123,29 @@ struct _PrefsCommon
 	gint compose_width;
 	gint compose_height;
 
+	/* Message */
+	gboolean enable_color;
+	gint quote_level1_col;
+	gint quote_level2_col;
+	gint quote_level3_col;
+	gint uri_col;
+	gushort sig_col;
+	gboolean recycle_quote_colors;
+	gboolean conv_mb_alnum;
+	gboolean display_header_pane;
+	gboolean display_header;
+	gboolean head_space;
+	gint line_space;
+	gboolean enable_smooth_scroll;
+	gint scroll_step;
+	gboolean scroll_halfpage;
+
+	gchar *force_charset;
+
+	/* MIME viewer */
+	gchar *mime_image_viewer;
+	gchar *mime_audio_player;
+
 	/* Privacy */
 	gboolean default_encrypt;
 	gboolean default_sign;
@@ -150,12 +160,12 @@ struct _PrefsCommon
 	gboolean open_unread_on_enter;
 	gboolean open_inbox_on_inc;
 	gboolean immediate_exec;
-	gboolean enable_smooth_scroll;
-	gint scroll_step;
+	gboolean add_address_by_click;
 
 	gboolean confirm_on_exit;
 	gboolean clean_on_exit;
 	gboolean ask_on_clean;
+	gboolean warn_queued_on_exit;
 
 	/* Other */
 	gchar *uri_cmd;
