@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "procheader.h"
 #include "procmsg.h"
 #include "codeconv.h"
+#include "prefs_common.h"
 #include "utils.h"
 
 #define BUFFSIZE	8192
@@ -523,24 +524,13 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 
 void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 {
-	static gchar *wdaystr = N_("SunMonTueWedThuFriSat");
-	static gchar *tr_wday = NULL;
 	struct tm *lt;
-	gint wdlen;
-	gchar *wday;
-
-	if (!tr_wday)
-		tr_wday = g_strdup(gettext(wdaystr));
+	gchar *default_format = "%y/%m/%d(%a) %H:%M";
 
 	lt = localtime(&timer);
 
-	wdlen = strlen(tr_wday) / 7;
-	if ((wday = alloca(wdlen + 1))) {
-		strncpy(wday, tr_wday + lt->tm_wday * wdlen, wdlen);
-		wday[wdlen] = '\0';
-	}
-
-	g_snprintf(dest, len, "%02d/%d/%d(%s) %02d:%02d",
-		   lt->tm_year % 100, lt->tm_mon + 1, lt->tm_mday,
-		   wday, lt->tm_hour, lt->tm_min);
+	if (prefs_common.date_format)
+		strftime(dest, len, prefs_common.date_format, lt);
+	else
+		strftime(dest, len, default_format, lt);
 }
