@@ -71,7 +71,8 @@ void procmime_mimeinfo_free(MimeInfo *mimeinfo)
 	g_free(mimeinfo->filename);
 #if USE_GPGME
 	g_free(mimeinfo->plaintextfile);
-        g_free(mimeinfo->sigstatus);
+	g_free(mimeinfo->sigstatus);
+	g_free(mimeinfo->sigstatus_full);
 #endif
 
 	procmime_mimeinfo_free(mimeinfo->sub);
@@ -94,6 +95,7 @@ void procmime_mimeinfo_free_all(MimeInfo *mimeinfo)
 #if USE_GPGME
 		g_free(mimeinfo->plaintextfile);
 		g_free(mimeinfo->sigstatus);
+		g_free(mimeinfo->sigstatus_full);
 #endif
 
 		procmime_mimeinfo_free_all(mimeinfo->sub);
@@ -179,7 +181,8 @@ MimeInfo *procmime_scan_message(MsgInfo *msginfo)
 	}
 
 #if USE_GPGME
-	rfc2015_check_signature(mimeinfo, fp);
+        if (prefs_common.auto_check_signatures)
+		rfc2015_check_signature(mimeinfo, fp);
 #endif
 	fclose(fp);
 
@@ -520,7 +523,7 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 			guchar *p = buf;
 
 			softline = DoOneQPLine(&p, FALSE, softline);
-			fputs(buf, outfp);
+			fwrite(buf, p - (guchar *)buf, 1, outfp);
 		}
 	} else if (mimeinfo->encoding_type == ENC_BASE64) {
 		gchar outbuf[BUFFSIZE];
