@@ -32,17 +32,20 @@ typedef struct _MsgInfo		MsgInfo;
 
 typedef enum
 {
+	/* permanent flags (0x0000ffff) */
 	MSG_NEW		= 1 << 0,
 	MSG_UNREAD	= 1 << 1,
 	MSG_MARKED	= 1 << 2,
 	MSG_DELETED	= 1 << 3,
-	MSG_IMPORTANT	= 1 << 4,
-	MSG_MOVE	= 1 << 5,
-	MSG_COPY	= 1 << 6,
+	MSG_REPLIED	= 1 << 4,
+	MSG_FORWARDED	= 1 << 5,
 
-	MSG_QUEUED	= 1 << 16,
-	MSG_DRAFT	= 1 << 17,
+	/* temporary flags (0xffff0000) */
+	MSG_MOVE	= 1 << 16,
+	MSG_COPY	= 1 << 17,
 
+	MSG_QUEUED	= 1 << 25,
+	MSG_DRAFT	= 1 << 26,
 	MSG_ENCRYPTED   = 1 << 27,
 	MSG_IMAP	= 1 << 28,
 	MSG_MIME	= 1 << 29,
@@ -50,15 +53,26 @@ typedef enum
 	MSG_CACHED	= 1 << 31
 } MsgFlags;
 
+#define MSG_PERMANENT_FLAG_MASK		(MSG_NEW       | \
+					 MSG_UNREAD    | \
+					 MSG_MARKED    | \
+					 MSG_DELETED   | \
+					 MSG_REPLIED   | \
+					 MSG_FORWARDED)
+#define MSG_CACHED_FLAG_MASK		(MSG_MIME)
+
 #define MSG_SET_FLAGS(msg, flags)	{ (msg) |= (flags); }
 #define MSG_UNSET_FLAGS(msg, flags)	{ (msg) &= ~(flags); }
 #define MSG_IS_NEW(msg)			((msg & MSG_NEW) != 0)
 #define MSG_IS_UNREAD(msg)		((msg & MSG_UNREAD) != 0)
 #define MSG_IS_MARKED(msg)		((msg & MSG_MARKED) != 0)
 #define MSG_IS_DELETED(msg)		((msg & MSG_DELETED) != 0)
-#define MSG_IS_IMPORTANT(msg)		((msg & MSG_IMPORTANT) != 0)
+#define MSG_IS_REPLIED(msg)		((msg & MSG_REPLIED) != 0)
+#define MSG_IS_FORWARDED(msg)		((msg & MSG_FORWARDED) != 0)
+
 #define MSG_IS_MOVE(msg)		((msg & MSG_MOVE) != 0)
 #define MSG_IS_COPY(msg)		((msg & MSG_COPY) != 0)
+
 #define MSG_IS_QUEUED(msg)		((msg & MSG_QUEUED) != 0)
 #define MSG_IS_DRAFT(msg)		((msg & MSG_DRAFT) != 0)
 #define MSG_IS_ENCRYPTED(msg)		((msg & MSG_ENCRYPTED) != 0)
@@ -134,13 +148,14 @@ FILE   *procmsg_open_mark_file		(const gchar	*folder,
 					 gboolean	 append);
 
 void	procmsg_move_messages		(GSList		*mlist);
+void	procmsg_copy_messages		(GSList		*mlist);
 
 gchar  *procmsg_get_message_file_path	(MsgInfo	*msginfo);
 gchar  *procmsg_get_message_file	(MsgInfo	*msginfo);
 FILE   *procmsg_open_message		(MsgInfo	*msginfo);
 gboolean procmsg_msg_exist		(MsgInfo	*msginfo);
 
-void	procmsg_clean_trash		(void);
+void	procmsg_empty_trash		(void);
 gint	procmsg_send_queue		(void);
 void	procmsg_print_message		(MsgInfo	*msginfo,
 					 const gchar	*cmdline);
