@@ -33,7 +33,10 @@
 
 #define BUFFSIZE	8192
 
-gint recv_write_to_file(gint sock, const gchar *filename)
+static RecvUIFunc	recv_ui_func;
+static gpointer		recv_ui_func_data;
+
+gint recv_write_to_file(SockInfo *sock, const gchar *filename)
 {
 	FILE *fp;
 
@@ -63,7 +66,7 @@ gint recv_write_to_file(gint sock, const gchar *filename)
 	return 0;
 }
 
-gint recv_bytes_write_to_file(gint sock, glong size, const gchar *filename)
+gint recv_bytes_write_to_file(SockInfo *sock, glong size, const gchar *filename)
 {
 	FILE *fp;
 
@@ -93,7 +96,7 @@ gint recv_bytes_write_to_file(gint sock, glong size, const gchar *filename)
 	return 0;
 }
 
-gint recv_write(gint sock, FILE *fp)
+gint recv_write(SockInfo *sock, FILE *fp)
 {
 	gchar buf[BUFFSIZE];
 	gint len;
@@ -103,7 +106,7 @@ gint recv_write(gint sock, FILE *fp)
 	if (nb) sock_set_nonblocking_mode(sock, FALSE);
 
 	for (;;) {
-		if (sock_read(sock, buf, sizeof(buf)) < 0) {
+		if (sock_gets(sock, buf, sizeof(buf)) < 0) {
 			g_warning(_("error occurred while retrieving data.\n"));
 			if (nb) sock_set_nonblocking_mode(sock, TRUE);
 			return -1;
@@ -112,16 +115,20 @@ gint recv_write(gint sock, FILE *fp)
 		len = strlen(buf);
 		if (len > 1 && buf[0] == '.' && buf[1] == '\r') break;
 
+		if (recv_ui_func)
+			recv_ui_func(sock, len, recv_ui_func_data);
+
 		if (len > 1 && buf[len - 1] == '\n' && buf[len - 2] == '\r') {
 			buf[len - 2] = '\n';
 			buf[len - 1] = '\0';
+			len--;
 		}
 
 		if (buf[0] == '.' && buf[1] == '.')
-			memmove(buf, buf + 1, strlen(buf));
+			memmove(buf, buf + 1, len--);
 
 		if (!strncmp(buf, ">From ", 6))
-			memmove(buf, buf + 1, strlen(buf));
+			memmove(buf, buf + 1, len--);
 
 		if (fp && fputs(buf, fp) == EOF) {
 			perror("fputs");
@@ -137,7 +144,7 @@ gint recv_write(gint sock, FILE *fp)
 	return 0;
 }
 
-gint recv_bytes_write(gint sock, glong size, FILE *fp)
+gint recv_bytes_write(SockInfo *sock, glong size, FILE *fp)
 {
 	gchar *buf;
 	gboolean nb;
@@ -150,9 +157,9 @@ gint recv_bytes_write(gint sock, glong size, FILE *fp)
 	Xalloca(buf, size, return -1);
 
 	do {
-		size_t read_count;
+		gint read_count;
 
-		read_count = read(sock, buf + count, size - count);
+		read_count = sock_read(sock, buf + count, size - count);
 		if (read_count < 0) {
 			if (nb) sock_set_nonblocking_mode(sock, TRUE);
 			return -1;
@@ -160,19 +167,27 @@ gint recv_bytes_write(gint sock, glong size, FILE *fp)
 		count += read_count;
 	} while (count < size);
 
+	/* +------------------+----------------+--------------------------+ *
+	 * ^buf               ^prev            ^cur             buf+size-1^ */
+
 	prev = buf;
-	while ((cur = memchr(prev, '\r', size)) != NULL) {
-		if (cur - buf + 1 < size && *(cur + 1) == '\n') {
-			if (fwrite(prev, sizeof(gchar), cur - prev, fp) == EOF ||
-			    fwrite("\n", sizeof(gchar), 1, fp) == EOF) {
-				perror("fwrite");
-				g_warning(_("Can't write to file.\n"));
-				if (nb) sock_set_nonblocking_mode(sock, TRUE);
-				return -1;
-			}
-			prev = cur + 2;
-			if (prev - buf >= size) break;
+	while ((cur = memchr(prev, '\r', size - (prev - buf))) != NULL) {
+		if (cur == buf + size - 1) break;
+
+		if (fwrite(prev, sizeof(gchar), cur - prev, fp) == EOF ||
+		    fwrite("\n", sizeof(gchar), 1, fp) == EOF) {
+			perror("fwrite");
+			g_warning(_("Can't write to file.\n"));
+			if (nb) sock_set_nonblocking_mode(sock, TRUE);
+			return -1;
 		}
+
+		if (*(cur + 1) == '\n')
+			prev = cur + 2;
+		else
+			prev = cur + 1;
+
+		if (prev - buf >= size) break;
 	}
 
 	if (prev - buf < size && fwrite(buf, sizeof(gchar),
@@ -186,3 +201,9 @@ gint recv_bytes_write(gint sock, glong size, FILE *fp)
 	if (nb) sock_set_nonblocking_mode(sock, TRUE);
 	return 0;
 }
+
+void recv_set_ui_func(RecvUIFunc func, gpointer data)
+{
+	recv_ui_func = func;
+	recv_ui_func_data = data;
+}
