@@ -808,7 +808,7 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 		textview->text_is_mb = TRUE;
 	}
 
-	if (prefs_common.messagefont) {
+	if (prefs_common.textfont) {
 		if (textview->msgfont) {
 			textview->msgfont->ascent = textview->prev_ascent;
 			textview->msgfont->descent = textview->prev_descent;
@@ -817,14 +817,14 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 		}
 		if (use_fontset)
 			textview->msgfont =
-				gdk_fontset_load(prefs_common.messagefont);
+				gdk_fontset_load(prefs_common.textfont);
 		else {
 			if (MB_CUR_MAX > 1) {
 				FONT_LOAD(textview->msgfont,
 					  "-*-courier-medium-r-normal--14-*-*-*-*-*-iso8859-1");
 			} else {
 				FONT_LOAD(textview->msgfont,
-					  prefs_common.messagefont);
+					  prefs_common.textfont);
 			}
 		}
 
@@ -978,14 +978,20 @@ gboolean textview_scroll_page(TextView *textview, gboolean up)
 {
 	GtkText *text = GTK_TEXT(textview->text);
 	gfloat upper;
+	gfloat page_incr;
 
 	if (prefs_common.enable_smooth_scroll)
 		return textview_smooth_scroll_page(textview, up);
 
+	if (prefs_common.scroll_halfpage)
+		page_incr = text->vadj->page_increment / 2;
+	else
+		page_incr = text->vadj->page_increment;
+
 	if (!up) {
 		upper = text->vadj->upper - text->vadj->page_size;
 		if (text->vadj->value < upper) {
-			text->vadj->value += text->vadj->page_increment;
+			text->vadj->value += page_incr;
 			text->vadj->value = MIN(text->vadj->value, upper);
 			gtk_signal_emit_by_name(GTK_OBJECT(text->vadj),
 						"value_changed");
@@ -993,7 +999,7 @@ gboolean textview_scroll_page(TextView *textview, gboolean up)
 			return FALSE;
 	} else {
 		if (text->vadj->value > 0.0) {
-			text->vadj->value -= text->vadj->page_increment;
+			text->vadj->value -= page_incr;
 			text->vadj->value = MAX(text->vadj->value, 0.0);
 			gtk_signal_emit_by_name(GTK_OBJECT(text->vadj),
 						"value_changed");
@@ -1072,15 +1078,20 @@ static gboolean textview_smooth_scroll_page(TextView *textview, gboolean up)
 {
 	GtkText *text = GTK_TEXT(textview->text);
 	gfloat upper;
+	gfloat page_incr;
 	gfloat old_value;
 	gfloat last_value;
 
+	if (prefs_common.scroll_halfpage)
+		page_incr = text->vadj->page_increment / 2;
+	else
+		page_incr = text->vadj->page_increment;
+
 	if (!up) {
 		upper = text->vadj->upper - text->vadj->page_size;
 		if (text->vadj->value < upper) {
 			old_value = text->vadj->value;
-			last_value = text->vadj->value +
-				text->vadj->page_increment / 2;
+			last_value = text->vadj->value + page_incr;
 			last_value = MIN(last_value, upper);
 
 			textview_smooth_scroll_do(textview, old_value,
@@ -1091,8 +1102,7 @@ static gboolean textview_smooth_scroll_page(TextView *textview, gboolean up)
 	} else {
 		if (text->vadj->value > 0.0) {
 			old_value = text->vadj->value;
-			last_value = text->vadj->value -
-				text->vadj->page_increment / 2;
+			last_value = text->vadj->value - page_incr;
 			last_value = MAX(last_value, 0.0);
 
 			textview_smooth_scroll_do(textview, old_value,
